package org.apidesign.apifest08.currency;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * This class encapsulates settings for currency conversion
 * We can use it eg. for uptime options etc. in future versions
 *
 * @author Ladislav Vitasek
 */
final public class ConversionProperties {

    private final ConversionRatioProvider conversionRatioProvider;
    private final RoundingMode roundingMode;

    /**
     * Returns conversion constants - conversion operation is invertable
     * Default
     */
    private final static class FixedBidirectionalRatioConversion implements ConversionRatioProvider {
        private final BigDecimal currency1ToCurrency2Constant;
        private final BigDecimal currency2ToCurrency1Constant;
        private static final int SCALE_MAX_DEFAULT = 20;

        /**
         * Constructor
         * @param ratioConstant fixed ratio constant
         * @param mode math rounding mode
         * @throws IllegalArgumentException if conversion value is <= 0
         */
        private FixedBidirectionalRatioConversion(BigDecimal ratioConstant, RoundingMode mode) {
            if (ratioConstant.compareTo(BigDecimal.ZERO) <= 0)
                throw new IllegalArgumentException("Conversion value cannot be <= 0");
            this.currency1ToCurrency2Constant = ratioConstant;
            this.currency2ToCurrency1Constant = BigDecimal.ONE.setScale(SCALE_MAX_DEFAULT).divide(ratioConstant, mode);
        }

        public BigDecimal getCurrency1ToCurrency2Constant() {
            return currency1ToCurrency2Constant;
        }

        public BigDecimal getCurrency2ToCurrency1Constant() {
            return currency2ToCurrency1Constant;
        }
    }


    /**
     * Returns new instance Conversion properties - fixed conversion ratio - both directions
     * Default max scale is set to 20
     *
     * @param conversionConstant value of constant for conversion
     * @param roundingMode       math rounding mode
     * @return new instance of ConversionProperties class
     * @throws IllegalArgumentException if conversion constant is <= 0 
     * @see org.apidesign.apifest08.currency.ConversionProperties.FixedBidirectionalRatioConversion
     */
    public static ConversionProperties create(BigDecimal conversionConstant, RoundingMode roundingMode) {
        return create(new FixedBidirectionalRatioConversion(conversionConstant, roundingMode), roundingMode);
    }

    /**
     * Returns new instance Conversion properties
     *
     * @param conversionRatioProvider provider for conversion constants
     * @param roundingMode       math rounding mode
     * @return new instance of ConversionProperties class
     */
    public static ConversionProperties create(ConversionRatioProvider conversionRatioProvider, RoundingMode roundingMode) {
        if (conversionRatioProvider == null || roundingMode == null)
            throw new NullPointerException();
        return new ConversionProperties(conversionRatioProvider, roundingMode);
    }

    /**
     * Returns new instance Conversion properties with values set to conversionConstant=BigDecimal.ONE
     * and roundingMode=RoundingMode.HALF_EVEN
     * 
     * Conversion is bidirectional.
     *
     * @return
     */
    public static ConversionProperties createDefault() {
        return create(BigDecimal.ONE, RoundingMode.HALF_EVEN);
    }

    private ConversionProperties(ConversionRatioProvider conversionRatioProvider, RoundingMode roundingMode) {
        this.conversionRatioProvider = conversionRatioProvider;
        this.roundingMode = roundingMode;
    }


    public RoundingMode getRoundingMode() {
        return roundingMode;
    }

    public ConversionRatioProvider getConversionRatioProvider() {
        return conversionRatioProvider;
    }
}
